from bluer_ai.help.generic import help_functions as generic_help_functions

from bluer_flow import ALIAS
from bluer_flow.help.localflow import help_functions as help_localflow
from bluer_flow.help.workflow import help_functions as help_workflow

help_functions = generic_help_functions(plugin_name=ALIAS)


help_functions.update(
    {
        "localflow": help_localflow,
        "workflow": help_workflow,
    }
)



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from bluer_flow import NAME
from bluer_flow.workflow.patterns import list_of_patterns
from bluer_flow.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="list",
)
parser.add_argument(
    "--delim",
    type=str,
    default="+",
)
parser.add_argument(
    "--count",
    type=int,
    default=-1,
)
parser.add_argument(
    "--offset",
    type=int,
    default=0,
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = False
if args.task == "list":
    success = True

    output = list_of_patterns()[args.offset :]

    if args.count != -1:
        output = output[: args.count]

    print(delim.join(output))
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from bluer_options.env import load_config, load_env, get_env

load_env(__name__)
load_config(__name__)


BLUER_FLOW_DEFAULT_WORKFLOW_PATTERN = get_env("BLUER_FLOW_DEFAULT_WORKFLOW_PATTERN")

LOCALFLOW_SLEEP_BETWEEN_JOBS = get_env("LOCALFLOW_SLEEP_BETWEEN_JOBS", 3)



from typing import List
from enum import Enum, auto


class RunnerType(Enum):
    GENERIC = auto()
    LOCAL = auto()
    LOCALFLOW = auto()


def list_of_runners() -> List[str]:
    return sorted([runner_type.name.lower() for runner_type in RunnerType])



NAME = "bluer_flow"

ICON = "📜"

DESCRIPTION = f"{ICON} Workflow management."

VERSION = "5.57.1"

REPO_NAME = "bluer-flow"

MARQUEE = "https://github.com/kamangir/assets/raw/main/bluer_flow-local-map-reduce/workflow.gif?raw=true"

ALIAS = "@flow"


def fullname() -> str:
    return f"{NAME}-{VERSION}"



import os

from bluer_options.help.functions import get_help
from bluer_objects import file, README

from bluer_flow import NAME, VERSION, ICON, REPO_NAME
from bluer_flow.workflow.README import items as workflow_items
from bluer_flow.workflow.patterns import list_of_patterns
from bluer_flow.help.functions import help_functions


def build():
    return all(
        README.build(
            items=readme.get("items", []),
            cols=len(list_of_patterns()) + 1,
            path=os.path.join(file.path(__file__), readme["path"]),
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
            help_function=lambda tokens: get_help(
                tokens,
                help_functions,
                mono=True,
            ),
        )
        for readme in [
            {"items": workflow_items, "path": ".."},
            # aliases
            {"path": "docs/aliases/localflow.md"},
            {"path": "docs/aliases/workflow.md"},
        ]
    )



from typing import List

from bluer_options.terminal import show_usage, xtra

from bluer_flow.workflow.runners import list_of_runners
from bluer_flow.workflow.patterns import list_of_patterns


def help_create(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "pattern=<pattern>",
            xtra(",~upload", mono=mono),
        ]
    )

    args = ["[--publish_as <public-object-name>]"]

    return show_usage(
        [
            "workflow",
            "create",
            f"[{options}]",
            "[.|<job-name>]",
        ]
        + args,
        "create a <pattern> workflow.",
        {
            "pattern: {}".format(" | ".join(list_of_patterns())): "",
        },
        mono=mono,
    )


def help_monitor(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "~download,node=<node>,~upload"

    return show_usage(
        [
            "workflow",
            "monitor",
            f"[{options}]",
            "[.|<job-name>]",
            "[<command-line>]",
        ],
        "monitor <job-name>/workflow and run <command-line>.",
        mono=mono,
    )


def submit_options(
    mono: bool,
    cascade: bool = False,
):
    return "".join(
        (
            [
                xtra("~download,", mono=mono),
            ]
            if not cascade
            else []
        )
        + [
            xtra("dryrun,", mono=mono),
            "to=<runner>",
        ]
        + (
            [
                xtra(",~upload", mono=mono),
            ]
            if not cascade
            else []
        )
    )


runner_details = {
    "runner: {}".format(" | ".join(list_of_runners())): "",
}


def help_submit(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "workflow",
            "submit",
            f"[{submit_options(mono=mono)}]",
            "[.|<job-name>]",
        ],
        "submit workflow.",
        runner_details,
        mono=mono,
    )


help_functions = {
    "create": help_create,
    "monitor": help_monitor,
    "submit": help_submit,
}



from typing import Tuple, List
import os
import glob
import networkx as nx

from bluer_objects import file

from bluer_flow.workflow import dot_file


def list_of_patterns() -> List[str]:
    return sorted(
        [
            file.name(filename)
            for filename in glob.glob(
                os.path.join(
                    file.path(__file__),
                    "*.dot",
                )
            )
        ]
    )


def load_pattern(
    pattern: str,
    **kw_args,
) -> Tuple[bool, nx.DiGraph]:
    return dot_file.load_from_file(
        filename=os.path.join(file.path(__file__), f"{pattern}.dot"),
        **kw_args,
    )



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit
from bluer_objects.metadata import get_from_object, post_to_object

from bluer_flow.workflow.runners import RunnerType, list_of_runners
from bluer_flow.workflow.generic import Workflow
from bluer_flow import NAME
from bluer_flow.workflow.runners.factory import runner_class
from bluer_flow.workflow.runners.generic import GenericRunner
from bluer_flow.logger import logger

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="list|monitor|submit",
)
parser.add_argument(
    "--count",
    type=int,
    default=-1,
)
parser.add_argument(
    "--delim",
    type=str,
    default="+",
)

parser.add_argument(
    "--dryrun",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--job_name",
    type=str,
    default="",
)
parser.add_argument(
    "--hot_node",
    type=str,
    default="void",
)
parser.add_argument(
    "--offset",
    type=int,
    default=0,
)
parser.add_argument(
    "--runner_type",
    type=str,
    default="local",
    help="|".join(list_of_runners()),
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = False
if args.task == "list":
    success = True

    output = list_of_runners()[args.offset :]

    if args.count != -1:
        output = output[: args.count]

    print(delim.join(output))
elif args.task == "monitor":
    workflow = Workflow(
        job_name=args.job_name,
        load=True,
    )

    runner_type: str = get_from_object(
        args.job_name,
        "submission.runner_type",
    )

    runner: GenericRunner = (
        runner_class[RunnerType[runner_type.upper()]]()
        if runner_type.upper() in RunnerType.__members__
        else GenericRunner()
    )

    success = runner.monitor(workflow, args.hot_node)

    if success:
        success = workflow.save(caption=f"@{runner_type}")
elif args.task == "submit":
    workflow = Workflow(
        job_name=args.job_name,
        load=True,
    )

    runner = runner_class[RunnerType[args.runner_type.upper()]]()

    success = runner.submit(
        workflow,
        dryrun=args.dryrun == 1,
    )

    if success:
        success = workflow.save()

    if success:
        success = post_to_object(
            args.job_name,
            "submission.runner_type",
            args.runner_type,
        )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_eval(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("type=<cpu|gpu>,verbose", mono=mono)

    return show_usage(
        [
            "localflow",
            "eval",
            f"[{options}]",
            "<command-line>",
        ],
        "<command-line> -> localflow",
        mono=mono,
    )


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("status=<status>", mono=mono)

    return show_usage(
        [
            "localflow",
            "list",
            f"[{options}]",
        ],
        "list localflow jobs.",
        mono=mono,
    )


def help_rm(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("status=<status>", mono=mono)

    return show_usage(
        [
            "localflow",
            "rm",
            f"[{options}]",
        ],
        "rm localflow jobs.",
        mono=mono,
    )


def help_start(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("exit_if_no_job", mono=mono)

    return show_usage(
        [
            "localflow",
            "start",
            f"[{options}]",
        ],
        "start localflow",
        mono=mono,
    )


def help_stop(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "localflow",
            "stop",
        ],
        "stop localflow",
        mono=mono,
    )


help_functions = {
    "eval": help_eval,
    "list": help_list,
    "rm": help_rm,
    "start": help_start,
    "stop": help_stop,
}



import copy
import textwrap
from typing import Tuple, Dict, List
import numpy as np
import networkx as nx
from networkx.drawing.nx_pydot import write_dot
import matplotlib.pyplot as plt
from networkx.drawing.nx_pydot import read_dot
import textwrap

from blueness import module
from bluer_options.logger import crash_report
from bluer_objects import file, path

from bluer_flow import NAME
from bluer_flow.host import signature
from bluer_flow.logger import logger

NAME = module.name(__file__, NAME)


layouts = {
    "spring": nx.spring_layout,
    "circular": nx.circular_layout,
    "random": nx.random_layout,
    "shell": nx.shell_layout,
    "kamada_kawai": nx.kamada_kawai_layout,
    "spectral": nx.spectral_layout,
    "planar": nx.planar_layout,
    "fruchterman_reingold": nx.fruchterman_reingold_layout,
    "spiral": nx.spiral_layout,
}

status_color_map = {
    "SUBMITTED": "yellow",
    "PENDING": "orange",
    "RUNNABLE": "purple",
    "STARTING": "cyan",
    "RUNNING": "blue",
    "SUCCEEDED": "green",
    "FAILED": "black",
}


def export_graph_as_image(
    G: nx.DiGraph,
    filename: str,
    layout: str = "shell",
    figsize: int = 10,
    log: bool = True,
    colormap: Dict[str, str] = {},
    hot_node: str = "void",
    add_legend: bool = True,
    caption: str = "",
    text_width: int = 80,
    node_size: int = 200,
    font_size: int = 10,
) -> bool:
    layout_func = layouts.get(layout, None)
    if layout_func is None:
        logger.error(f"unknown layout: {layout}")
        return False

    if add_legend:
        G = copy.deepcopy(G)
        for status in status_color_map:
            G.add_node(status)
            G.nodes[status]["status"] = status

    pos = layout_func(G)

    node_color = [
        (
            "red"
            if node == hot_node
            else colormap.get(
                G.nodes[node].get("status"),
                "gray",
            )
        )
        for node in G
    ]

    plt.figure(figsize=(figsize, figsize))
    nx.draw(
        G,
        pos,
        with_labels=True,
        node_color=node_color,
        edge_color="gray",
        node_size=node_size,
        font_size=font_size,
        font_color="darkred",
        # arrowstyle="-|>",
        # arrowsize=5,
    )

    caption_items: List[str] = [caption]
    if hot_node in G.nodes:
        caption_items += [G.nodes[hot_node].get("command_line").replace('"', "")]
    caption_items = [item for item in caption_items if item]

    caption_wrapped = textwrap.fill(
        " | ".join(caption_items),
        width=text_width,
    )

    if caption_items:
        plt.text(
            0.5,
            0.5,
            caption_wrapped,
            horizontalalignment="center",
            verticalalignment="center",
            transform=plt.gca().transAxes,
            fontsize=12,
            color="green",
        )

    header = " | ".join(
        [
            path.name(file.path(filename)),
            file.name(filename),
            f"{G.number_of_nodes()} node(s)",
            f"{G.number_of_edges()} edge(s)",
            f"layout: {layout}",
            f"@{hot_node}",
        ]
    )
    plt.title(header, fontsize=10)
    plt.figtext(
        0.5,
        0.01,
        "\n".join(
            textwrap.wrap(
                " | ".join(signature()),
                width=len(header),
            ),
        ),
        ha="center",
        fontsize=10,
        color="black",
    )
    return file.save_fig(filename, log=log)


def load_from_file(
    filename: str,
    export_as_image: str = "",
    log: bool = True,
    **kwargs,
) -> Tuple[bool, nx.DiGraph]:
    G = nx.DiGraph()

    try:
        G = read_dot(filename)
    except:
        crash_report(f"-{NAME}: load_from_file({filename}): failed.")
        return False, G

    if log:
        logger.info(f"loaded {G} from {filename}.")

    return (
        not export_as_image
        or export_graph_as_image(
            G,
            filename=(
                file.add_extension(filename, export_as_image[1:])
                if export_as_image.startswith(".")
                else export_as_image
            ),
            log=log,
            **kwargs,
        )
    ), G


def save_to_file(
    filename: str,
    G: nx.DiGraph,
    export_as_image: str = ".png",
    log: bool = True,
    **kwargs,
) -> bool:
    if not file.prepare_for_saving(filename):
        return False

    try:
        write_dot(G, filename)
    except:
        crash_report(f"save_to_file({filename}) failed.")
        return False

    if log:
        logger.info(f"{G} -> {filename}")

    if not export_as_image:
        return True

    return export_graph_as_image(
        G,
        filename=(
            file.add_prefix(
                file.add_extension(
                    filename,
                    export_as_image[1:],
                ),
                "thumbnail",
            )
            if export_as_image.startswith(".")
            else export_as_image
        ),
        log=log,
        **kwargs,
    )



from functools import reduce

from bluer_options import string

from bluer_flow.workflow.patterns import list_of_patterns
from bluer_flow.workflow.runners import list_of_runners


items = (
    ["📜"]
    + [
        "[`{}`](./patterns/{}.dot)".format(
            pattern,
            pattern,
        )
        for pattern in list_of_patterns()
    ]
    + reduce(
        lambda x, y: x + y,
        [
            (
                [
                    (
                        f"[{runner_type}](./bluer_flow/workflow/runners/{runner_type}/runner.py)"
                        if runner_type == "localflow"
                        else f"[{runner_type}](./bluer_flow/workflow/runners/{runner_type}.py)"
                    )
                ]
                + [
                    f"[![image]({url})]({url}) [🔗]({url})"
                    for url in [
                        "https://github.com/kamangir/assets/blob/main/bluer_flow-{}-{}/workflow.gif?raw=true&random={}".format(
                            runner_type,
                            pattern,
                            string.random(),
                        )
                        for pattern in list_of_patterns()
                    ]
                ]
            )
            for runner_type in list_of_runners()
        ],
        [],
    )
)



from typing import Any, List, Tuple, Dict
import glob
from tqdm import tqdm

from blueness import module
from bluer_options import string
from bluer_objects import file, objects
from bluer_objects.graphics.gif import generate_animated_gif
from bluer_objects.metadata import post_to_object

from bluer_flow import NAME
from bluer_flow.logger import logger
from bluer_flow.workflow import dot_file
from bluer_flow.workflow.generic import Workflow


NAME = module.name(__file__, NAME)


class GenericRunner:
    def __init__(self):
        self.type_name: str = "generic"
        self.job_name: str = ""

    def monitor(
        self,
        workflow: Workflow,
        hot_node: str = "void",
    ) -> bool:
        self.job_name = workflow.job_name

        try:
            workflow = self.monitor_function(workflow, hot_node)
        except Exception as e:
            logger.warning(f"monitor failed: {e}")

        summary: Dict[str, str] = {}
        for node in workflow.G.nodes:
            summary.setdefault(workflow.G.nodes[node].get("status"), []).append(node)
        for status, nodes in summary.items():
            logger.info("{}: {}".format(status, ", ".join(sorted(nodes))))

        if not dot_file.export_graph_as_image(
            workflow.G,
            objects.path_of(
                "thumbnail-workflow-{}.png".format(
                    string.pretty_date(as_filename=True, unique=True),
                ),
                workflow.job_name,
            ),
            colormap=dot_file.status_color_map,
            hot_node=hot_node,
            caption=f"{workflow.name} @ {self.type_name}",
        ):
            return False

        return generate_animated_gif(
            [
                filename
                for filename in sorted(
                    glob.glob(
                        objects.path_of("thumbnail-workflow-*.png", workflow.job_name)
                    )
                )
                if len(file.name(filename)) > 15
            ],
            objects.path_of("workflow.gif", workflow.job_name),
            frame_duration=333,
        )

    def monitor_function(
        self,
        workflow: Workflow,
        hot_node: str,
    ) -> Workflow:
        logger.info(
            f"{NAME}.{self.__class__.__name__}.monitor: {workflow} @ {hot_node}"
        )

        post_to_object(
            workflow.node_job_name(hot_node),
            "monitor",
            {"hot_node": hot_node},
        )

        return workflow

    def submit(
        self,
        workflow: Workflow,
        dryrun: bool = True,
    ) -> bool:
        self.job_name = workflow.job_name

        logger.info(
            "{}.{}.submit({}, dryrun={})".format(
                NAME,
                self.__class__.__name__,
                workflow.G,
                dryrun,
            )
        )

        metadata: Dict[str, Any] = {}
        failure_count: int = 0
        round: int = 1
        while not all(
            workflow.G.nodes[node].get("job_id") for node in workflow.G.nodes
        ):
            for node in tqdm(workflow.G.nodes):
                if workflow.G.nodes[node].get("job_id"):
                    continue

                pending_dependencies = [
                    node_
                    for node_ in workflow.G.successors(node)
                    if not workflow.G.nodes[node_].get("job_id")
                ]
                if pending_dependencies:
                    logger.info(
                        '⏳ node "{}": {} pending dependenci(es): {}'.format(
                            node,
                            len(pending_dependencies),
                            ", ".join(pending_dependencies),
                        )
                    )
                    continue

                command_line = workflow.G.nodes[node]["command_line"]
                job_name = f"{workflow.job_name}-{node}"

                if dryrun:
                    workflow.G.nodes[node]["job_id"] = f"dryrun-round-{round}"
                    logger.info(f"{command_line} -> {job_name}")
                    continue

                success, metadata[node] = self.submit_command(
                    command_line=command_line,
                    job_name=job_name,
                    dependencies=[
                        workflow.G.nodes[node_].get("job_id")
                        for node_ in workflow.G.successors(node)
                    ],
                    verbose=False,
                    type=workflow.G.nodes[node].get("type", "cpu"),
                )
                if not success:
                    failure_count += 1

                workflow.G.nodes[node]["job_id"] = (
                    metadata[node]["job_id"] if success else "failed"
                )

            logger.info(f"end of round {round}")
            round += 1

        if failure_count:
            logger.error(f"{failure_count} failure(s).")

        if not workflow.save(caption=f"{self.__class__.__name__}.submit"):
            return False

        if not post_to_object(
            workflow.job_name,
            "submission",
            {
                "metadata": metadata,
                "failure_count": failure_count,
                "runner_type": self.type_name,
            },
        ):
            return False

        return failure_count == 0

    def submit_command(
        self,
        command_line: str,
        job_name: str,
        dependencies: List[str],
        verbose: bool = False,
        type: str = "cpu",
    ) -> Tuple[bool, Any]:
        logger.info(
            "⏳ {}.{}: {}[{}]: {} X {} dependency(s): {}".format(
                NAME,
                self.__class__.__name__,
                job_name,
                type,
                command_line,
                len(dependencies),
                ", ".join(dependencies),
            )
        )
        return True, {"job_id": job_name}



from typing import Any, List, Tuple
from tqdm import tqdm
from functools import reduce

from bluer_objects import file, objects

from bluer_flow.workflow.generic import Workflow
from bluer_flow.workflow.runners.generic import GenericRunner


class LocalRunner(GenericRunner):
    def __init__(self, **kw_args):
        super().__init__(**kw_args)
        self.type_name: str = "local"

        self.command_line_list: List[str] = []

    def monitor_function(
        self,
        workflow: Workflow,
        hot_node: str,
    ) -> Workflow:
        workflow = super().monitor_function(workflow, hot_node)

        for node in tqdm(workflow.G.nodes):
            if file.exists(
                objects.path_of(
                    "metadata.yaml",
                    workflow.node_job_name(node),
                )
            ):
                workflow.G.nodes[node]["status"] = "SUCCEEDED"

        return workflow

    def submit_command(
        self,
        command_line: str,
        job_name: str,
        dependencies: List[str],
        verbose: bool = False,
        type: str = "cpu",
    ) -> Tuple[bool, Any]:
        assert super().submit_command(
            command_line,
            job_name,
            dependencies,
            verbose,
            type,
        )[0]

        self.command_line_list += [command_line]

        filename = objects.path_of(f"{self.job_name}.sh", self.job_name)

        script = (
            [
                "#! /usr/bin/env bash",
                "",
                "function runme() {",
                '    echo "⏳ {} started: {} command(s)."'.format(
                    self.job_name,
                    len(self.command_line_list),
                ),
            ]
            + reduce(
                lambda x, y: x + y,
                [
                    [
                        "",
                        f"    bluer_ai_eval - {command_line}",
                        "    [[ $? -ne 0 ]] && return 1",
                    ]
                    for command_line in self.command_line_list
                ],
            )
            + [
                f'    echo "⏳ / {self.job_name}"',
                "}",
                "",
                'runme "$@"',
            ]
        )

        return file.save_text(filename, script, log=True), {"job_id": job_name}



from typing import Dict

from bluer_flow.workflow.runners import RunnerType
from bluer_flow.workflow.runners.local import LocalRunner
from bluer_flow.workflow.runners.localflow.runner import LocalFlowRunner
from bluer_flow.workflow.runners.generic import GenericRunner

runner_class: Dict[RunnerType, GenericRunner] = {
    RunnerType.GENERIC: GenericRunner,
    RunnerType.LOCAL: LocalRunner,
    RunnerType.LOCALFLOW: LocalFlowRunner,
}



from typing import Any, Dict, Any
import networkx as nx
from copy import deepcopy

from bluer_objects import objects, metadata, file

from bluer_flow import env
from bluer_flow.workflow import dot_file
from bluer_flow.workflow import patterns


class Workflow:
    def __init__(
        self,
        job_name: str = "",
        load: bool = False,
        name: str = "",
        args: Dict[str, Any] = {},
    ):
        self.job_name = job_name if job_name else objects.unique_object()

        self.name = name

        self.args = deepcopy(args)

        self.G: nx.DiGraph = nx.DiGraph()

        if load:
            success, self.G = dot_file.load_from_file(
                objects.path_of(
                    filename="workflow.dot",
                    object_name=self.job_name,
                )
            )
            assert success

            workflow_metadata = metadata.get_from_object(
                object_name=self.job_name,
                key="workflow",
                default={},
            )
            assert isinstance(workflow_metadata, dict)
            self.name = workflow_metadata.get("name", "not-found")
            self.args = workflow_metadata.get(
                "args",
                {"warning": "not-found"},
            )

    def get_metadata(self, key: str, default: str = "") -> str:
        return metadata.get_from_object(self.job_name, key, default)

    def load_pattern(
        self,
        pattern: str = env.BLUER_FLOW_DEFAULT_WORKFLOW_PATTERN,
        publish_as: str = "",
    ) -> bool:
        self.name = pattern
        self.args = {
            "pattern": pattern,
            "publish_as": publish_as,
        }

        success, self.G = patterns.load_pattern(
            pattern=pattern,
            export_as_image=objects.path_of(f"{pattern}.png", self.job_name),
        )
        if not success:
            return success

        self.G.add_node("X")
        for node in self.G.nodes:
            if self.G.in_degree(node) == 0 and node != "X":
                self.G.add_edge("X", node)

        for node in self.G.nodes:
            self.G.nodes[node]["command_line"] = (
                "bluer_flow_workflow monitor node={}{} {}".format(
                    node,
                    f",publish_as={publish_as}" if publish_as and node == "X" else "",
                    self.job_name,
                )
            )

        return self.post_metadata(
            "load_pattern",
            {"pattern": pattern},
        )

    def node_job_name(self, node: str) -> str:
        return f"{self.job_name}-{node}"

    def post_metadata(self, key: str, value: Any) -> bool:
        return metadata.post_to_object(self.job_name, key, value)

    def save(
        self,
        caption: str = "",
    ) -> bool:
        if not dot_file.save_to_file(
            objects.path_of(
                filename="workflow.dot",
                object_name=self.job_name,
            ),
            self.G,
            caption=" | ".join([item for item in [self.name, caption] if item]),
        ):
            return False

        return metadata.post_to_object(
            object_name=self.job_name,
            key="workflow",
            value={
                "name": self.name,
                "args": self.args,
            },
        )



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from bluer_flow import NAME
from bluer_flow.workflow.patterns import list_of_patterns
from bluer_flow.workflow.generic import Workflow
from bluer_flow.logger import logger


NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="create",
)
parser.add_argument(
    "--job_name",
    type=str,
    default="",
)
parser.add_argument(
    "--pattern",
    type=str,
    default=list_of_patterns()[0],
    help="|".join(list_of_patterns()),
)
parser.add_argument(
    "--publish_as",
    type=str,
    default="",
    help="<public-object-name>",
)
args = parser.parse_args()


success = False
if args.task == "create":
    workflow = Workflow(job_name=args.job_name)

    success = workflow.load_pattern(
        pattern=args.pattern,
        publish_as=args.publish_as,
    )

    if success:
        success = workflow.save()
else:
    success = None

sys_exit(logger, NAME, args.task, success)
